/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ThreadPool.java 5224 2009-11-08 14:00:33Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.thread;

import java.util.concurrent.Executor;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.TimeoutPoolException;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.waitcontrol.ExpireWaitControl;
import org.ow2.util.pool.impl.enhanced.impl.waitcontrol.InfiniteWaitControl;
import org.ow2.util.pool.impl.enhanced.impl.waitcontrol.NoWaitControl;

/**
 * @author Gael Lalire
 */
public class ThreadPool implements Executor {

    private static final Log LOG = LogFactory.getLog(ThreadPool.class);

    private IPool<IReusableThread> pool;

    private Long waitTimeBeforeUsingSelfThread;

    public ThreadPool(final IPool<IReusableThread> pool) {
        this(pool, null);
    }

    /**
     * @param pool
     * @param waitTimeBeforeUsingSelfThread if null then the threadPool will
     *        never use self thread
     */
    public ThreadPool(final IPool<IReusableThread> pool, final Long waitTimeBeforeUsingSelfThread) {
        this.pool = pool;
        this.waitTimeBeforeUsingSelfThread = waitTimeBeforeUsingSelfThread;
    }

    public void execute(final Runnable runnable) {
        boolean getCalled = false;
        IWaitControl waitControl;
        if (waitTimeBeforeUsingSelfThread == null) {
            waitControl = new InfiniteWaitControl();
        } else {
            long longValue = waitTimeBeforeUsingSelfThread.longValue();
            if (longValue <= 0) {
                waitControl = NoWaitControl.INSTANCE;
            } else {
                waitControl = new ExpireWaitControl(System.currentTimeMillis() + longValue);
            }
        }
        while (!getCalled) {
            try {
                pool.get(waitControl).setUsed(pool, runnable);
                getCalled = true;
            } catch (InterruptedException e) {
                LOG.warn("Interrupted", e);
            } catch (TimeoutPoolException e) {
                if (!waitControl.canContinueToWait()) {
                    runnable.run();
                    getCalled = true;
                } else {
                    LOG.warn("Unexpected exception", e);
                }
            } catch (PoolException e) {
                LOG.warn("PoolException", e);
            }
        }
    }

}
