package org.ow2.util.pool.impl.enhanced.impl.waitcontrol;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.WaiterInterruptedException;
import org.ow2.util.pool.impl.enhanced.internal.conditionWaitPart.IConditionWaitPart;

/**
 * @author Gael Lalire
 */
public class ExpireWaitControl implements IWaitControl {

    private long expireTime;

    private List<WaitAuthorization> waitAuthorizations;

    public ExpireWaitControl(final long expireTime) {
        this.expireTime = expireTime;
        waitAuthorizations = new ArrayList<WaitAuthorization>();
    }

    public void popWaitAuthorization() {
        waitAuthorizations.remove(0);
    }

    public void pushWaitAuthorization(final WaitAuthorization waitAuthorization) {
        waitAuthorizations.add(0, waitAuthorization);
    }

    protected boolean canWait() {
        for (WaitAuthorization waitAuthorization : waitAuthorizations) {
            if (!waitAuthorization.isWaitAllowed()) {
                return false;
            }
        }
        return true;
    }

    public boolean waitOnCondition(final Condition condition) throws InterruptedException, WaiterInterruptedException {
        long timeToWait = expireTime - System.currentTimeMillis();
        if (timeToWait <= 0) {
            return false;
        }
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        return condition.await(timeToWait, TimeUnit.MILLISECONDS);
    }

    public boolean waitOnMutex(final Object mutex) throws InterruptedException, WaiterInterruptedException {
        long timeToWait = expireTime - System.currentTimeMillis();
        if (timeToWait <= 0) {
            return false;
        }
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        mutex.wait(timeToWait);
        if (expireTime <= System.currentTimeMillis()) {
            return false;
        }
        return true;
    }

    public boolean canContinueToWait() {
        if (expireTime <= System.currentTimeMillis()) {
            return false;
        }
        return true;
    }

    public boolean waitOnConditionWaitPart(final IConditionWaitPart condition) throws InterruptedException,
            WaiterInterruptedException {
        long timeToWait = expireTime - System.currentTimeMillis();
        if (timeToWait <= 0) {
            return false;
        }
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        return condition.await(timeToWait, TimeUnit.MILLISECONDS);
    }

    public void verifyInterrupted() throws WaiterInterruptedException {
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
    }

}
